///////////////////////////////////////////////////////////////////////////////
//CS 441 - Distributed Object Programming - Fall 2004                        //
//MP 1                                                                       //
//                                                                           //
//Name : Jin Soon Lim                                                        //
//CS id: jlim                                                                //
//Due date: 09/21/2004                                                       //
///////////////////////////////////////////////////////////////////////////////

//source: Dbc.java                                                           //
//tested at bert.cs.uic.edu                                                  //

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.*;
import java.net.*;
import javax.swing.*;

public class Dbc extends JFrame implements ActionListener {
	public static final int PORT = 15703;

    String id;   // client id
    String host;
    Socket socket;
    ObjectInputStream in = null;
    ObjectOutputStream out = null;
    
    JTextArea textbox;
    JButton b_read, b_write, b_setrl, b_setwl, b_clr, b_exit;
    
    public Dbc(String host) {
            this.host = host;
        try {
            socket = new Socket(host, PORT);                         // connect to the host
            out = new ObjectOutputStream(socket.getOutputStream());  // set output stream
            in = new ObjectInputStream(socket.getInputStream());     // set input stream
            
            gui();  // create user interface
            println("Connected to " + host + ":" + PORT + ".");
            
            // wait for packets from the server
            while (true) {
                unpacker((Dbpacket)in.readObject());  // unpack the packet recieved
            }
        }
        catch (UnknownHostException e) {
            System.err.println(e.getMessage());
        }
        catch (IOException e) {
            System.err.println(e.getMessage());
        }
        catch (ClassNotFoundException e) {
            System.err.println(e.getMessage());
        }
        stop();  // stop the client
    }
    
    private void stop() {
        try {
            // close io streams and socket
            if (out != null) out.close();
            if (in != null) in.close();
            if (socket != null && !socket.isClosed()) socket.close();
            
            // disenable buttons
            b_read.setEnabled(false);
            b_write.setEnabled(false);
            b_setrl.setEnabled(false);
            b_setwl.setEnabled(false);
            b_clr.setEnabled(false);
            setTitle("Client " + id + " - Disconnected");
        }
        catch (IOException e) {
            System.err.println(e.getMessage());
        }
        catch (NullPointerException e) {
        }
    }
    
    private void exit() {
        sendPacket(new Dbpacket(Dbpacket.OP_CLIENT_EXIT));  // inform the server that the client will exit.
        stop();
        System.exit(0);
    }
    
    private void unpacker(Dbpacket p) {
        // process opcodes
        switch (p.getOpcode()) {
            // get client id from the server
            case Dbpacket.OP_CLIENT_INIT:
                id = p.getValue();
                setTitle("Client " + id + " - Connected");
                break;
            
            // print a message recieved from the server
            case Dbpacket.OP_SEND_MSG:
                println(p.getValue());
                break;
            
            // stop the client when the server is stopped
            case Dbpacket.OP_SERVER_SHUTDOWN:
                stop();
                println("The server is stopped.");
                break;
            
            default:
                println("Error: undefined packet");
        }
    }
    
    private void sendMsg(String str) {
        // send a message to the server
        Dbpacket p = new Dbpacket(Dbpacket.OP_SEND_MSG, str);
        sendPacket(p);
    }
    
    private void sendPacket(Dbpacket p) {
        // send a packet to the server
        try {
            out.writeObject(p);
            out.flush();
        }
        catch (IOException e) {
            System.err.println(e.getMessage());
        }
    }
    
    private void println(String str) {
        // print a string to the client text area
        textbox.append(str + "\n");
        textbox.setCaretPosition(textbox.getDocument().getLength()); 
    }
    
    private void gui() {
    	setSize(320, 240);
        JPanel mainpanel = new JPanel(new BorderLayout());
        
        // create a text area
        textbox = new JTextArea();
        textbox.setEditable(false);
        JScrollPane scrolltextbox = new JScrollPane(textbox);
        mainpanel.add(scrolltextbox);
        
        // create a tool bar
        JToolBar toolbar = new JToolBar("Client Tools");
        b_read = new JButton("Read");
        b_read.addActionListener(this);
        b_write = new JButton("Write");
        b_write.addActionListener(this);
        b_setrl = new JButton("SetRL");
        b_setrl.addActionListener(this);
        b_setwl = new JButton("SetWL");
        b_setwl.addActionListener(this);
        b_clr = new JButton("Clear");
        b_clr.addActionListener(this);
        b_exit = new JButton("Exit");
        b_exit.addActionListener(this);
        toolbar.add(b_read);
        toolbar.add(b_write);
        toolbar.addSeparator();
        toolbar.add(b_setrl);
        toolbar.add(b_setwl);
        toolbar.add(b_clr);
        toolbar.addSeparator();
        toolbar.add(b_exit);
        
        getContentPane().add(toolbar, BorderLayout.NORTH);
        getContentPane().add(mainpanel, BorderLayout.CENTER);
        setVisible(true);
    }
    
    public void actionPerformed (ActionEvent e) {
        // handle action events
        Object obj = e.getSource();
        
        if (obj == b_read) {
          	sendPacket(new Dbpacket(Dbpacket.OP_READ, getRID()));
        }
        else if (obj == b_write) {
            sendPacket(new Dbpacket(Dbpacket.OP_WRITE, getRID(), JOptionPane.showInputDialog(this, "Enter the new value.")));
        }
        else if (obj == b_setrl) {
            sendPacket(new Dbpacket(Dbpacket.OP_SET_RL, getRID()));
        }
        else if (obj == b_setwl) {
            sendPacket(new Dbpacket(Dbpacket.OP_SET_WL, getRID()));
        }
        else if (obj == b_clr) {
            sendPacket(new Dbpacket(Dbpacket.OP_CLEAR_LOCK, getRID()));
        }
        else if (obj == b_exit) {
            exit();
        }
    }
    
    private int getRID() {
        // get rid from user input
        int rid = -1;
        try {
            rid = Integer.parseInt(JOptionPane.showInputDialog(this, "Enter the record number."));
        }
        catch (NumberFormatException ex) {
        }
        return rid;  
    }
    
    protected void processWindowEvent(WindowEvent e) {
        // for processing program exit
        if (e.getID() == WindowEvent.WINDOW_CLOSING)
            exit();
        else
            super.processWindowEvent(e);
    }
    
    public static void main(String[] args) {
        if (args.length == 0)
            new Dbc("127.0.0.1");  // connect to the local server
        else
        	new Dbc(args[0]);
    }
} // end of Dbc class
